@@ -4,9 +4,9 @@
 
 ;; Author: Timm Lichte <timm.lichte@uni-tuebingen.de>
 ;; URL: https://codeberg.org/timmli/mu4e-walk
-;; Package-Version: 20251103.712
-;; Package-Revision: 8f5360aca1df
-;; Last modified: 2025-10-22 Wed 12:47:30
+;; Package-Version: 20251216.2015
+;; Package-Revision: 3d0b613cfdd5
+;; Last modified: 2025-12-16 Tue 10:12:20
 ;; Package-Requires: ((emacs "29.1"))
 ;; Keywords: convenience mail
 
@@ -31,15 +31,22 @@
 ;; - Move email address "vertically" between address fields
 ;; - Move email address "horizontally" within an address field
 ;; - Also works with regions
+;; - Moreover one can "vertically" and "horizontally" switch between email addresses.
+;; - Moreover one can delete an email address at point.
 
 ;; The most important functions and default keybindings are:
 
-;; | Function        | Keybinding  |
-;; |-----------------+-------------|
-;; | mu4e-walk-up    | M-<up>      |
-;; | mu4e-walk-down  | M-<down>    |
-;; | mu4e-walk-left  | M-<left>    |
-;; | mu4e-walk-right | M-<right>   |
+;; | Function                                | Keybinding  |
+;; |-----------------------------------------+-----------------------|
+;; | mu4e-walk-up                            | M-<up>                |
+;; | mu4e-walk-down                          | M-<down>              |
+;; | mu4e-walk-left                          | M-<left>              |
+;; | mu4e-walk-right                         | M-<right>             |
+;; | mu4e-walk-switch-up                     | C-<up>                |
+;; | mu4e-walk-switch-down                   | C-<down>              |
+;; | mu4e-walk-switch-left                   | C-<left>              |
+;; | mu4e-walk-switch-right                  | C-<right>             |
+;; | mu4e-walk-delete-email-address-at-point | M-<delete> M-<delete> |
 
 ;; Installation note: This package depends on mu4e which must be installed
 ;; separately from package.el.
@@ -58,6 +65,9 @@
 (defvar mu4e-walk-description+email-regexp "[[:space:]]*\\(\"[^\"]*\"[[:space:]]*<[-+_.~a-zA-Z][-+_.~:a-zA-Z0-9]*@[-.a-zA-Z0-9]+>\\|[^,:\"]*<?[-+_.~a-zA-Z][-+_.~:a-zA-Z0-9]*@[-.a-zA-Z0-9]+>?\\)"
   "Regular expression of an email address including optional description.")
 
+(defvar mu4e-walk-use-overlays t
+  "If non-nil, use overlays.")
+
 
 ;;====================
 ;;
@@ -76,46 +86,45 @@
 (defun mu4e-walk--email-address-at-point ()
   "Return the email address at point as a plist."
   (when (mu4e-walk--point-in-address-field-p)
-    (if (region-active-p)
-        `(:email ,(buffer-substring-no-properties (region-beginning) (region-end))
-                 :start ,(region-beginning)
-                 :end ,(region-end)
-                 :relpos ,(- (region-end) (region-beginning))
-                 :active t)
-      (let ((email)
-            (start)
-            (end)
-            (relpos)
-            (active nil))
-        (save-excursion
-          (let ((point (point)))
-            (cl-loop
-             while (re-search-backward "[,:]" (line-beginning-position) t)
-             do (setq start (+ (point) 1)
-                      end (save-excursion
-                            (re-search-forward mu4e-walk-description+email-regexp
-                                               (line-end-position) t)))
-             (when (<= point end)
-               (setq email (string-trim
-                            (string-replace
-                             "\n" ""
-                             (buffer-substring-no-properties start end))))
-               (when (string-match (concat "^" mu4e-walk-description+email-regexp "$")
-                                   email)
-                 (setq relpos (max 0 (- end point)))
-                 (cl-return `(:email ,email
-                                     :start ,start
-                                     :end ,end
-                                     :relpos ,relpos
-                                     :active ,active)))))))))))
+    (let ((email)
+          (start)
+          (end)
+          (relpos)
+          (active nil))
+      (save-excursion
+        (let ((point (point)))
+          (cl-loop
+           while (re-search-backward "[,:]" (line-beginning-position) t)
+           do (when (and (setq start (save-excursion
+                                       (when (re-search-forward
+                                              "\\([,:][[:space:]]*\\)[[:graph:]]" (line-end-position) t)
+                                         (match-end 1)))
+                               end (save-excursion
+                                     (re-search-forward mu4e-walk-description+email-regexp
+                                                        (line-end-position) t)))
+                         (<= point end))
+                (setq email (string-trim
+                             (string-replace
+                              "\n" ""
+                              (buffer-substring-no-properties start end))))
+                (when (string-match (concat "^" mu4e-walk-description+email-regexp "$")
+                                    email)
+                  (setq relpos (max 0 (- end point)))
+                  (cl-return `(:email ,email
+                                      :start ,start
+                                      :end ,end
+                                      :relpos ,relpos
+                                      :active ,active))))))))))
 
 (defun mu4e-walk--clean-address-field-at-point ()
   "Clean address field at point."
   (let ((start (line-beginning-position))
         (end (line-end-position)))
     (replace-regexp-in-region "\\([:,]\\)[ ]*," "\\1 " start end)
-    (replace-regexp-in-region "  " " " start end)
+    (replace-regexp-in-region "[, ]*,[, ]*" ", " start end)
+    (replace-regexp-in-region " [ ]+" " " start end)
     (replace-regexp-in-region ",[ ]*$" "" start end)
+    (replace-regexp-in-region "[ ]+$" "" start end)
     (replace-regexp-in-region ":$" ": " start end)))
 
 
@@ -128,7 +137,13 @@
 (defun mu4e-walk--move-email-address-at-point (&optional direction)
   "Move email address to previous or next address field.
 DIRECTION can be \\='up, \\='down, \\='left, \\='right."
-  (let* ((email-plist (mu4e-walk--email-address-at-point))
+  (let* ((email-plist (if (region-active-p)
+                          `(:email ,(buffer-substring-no-properties (region-beginning) (region-end))
+                                   :start ,(region-beginning)
+                                   :end ,(region-end)
+                                   :relpos ,(- (region-end) (region-beginning))
+                                   :active t)
+                        (mu4e-walk--email-address-at-point)))
          (email (plist-get email-plist :email))
          (start (plist-get email-plist :start))
          (end (plist-get email-plist :end))
@@ -231,6 +246,131 @@ DIRECTION can be \\='up, \\='down, \\='left, \\='right."
   (mu4e-walk--move-email-address-at-point 'right))
 
 
+;;====================
+;;
+;; Switching
+;;
+;;--------------------
+
+(defun mu4e-walk--switch (&optional direction)
+  "Switch to email address close by.
+DIRECTION can be \\='up, \\='down, \\='left, \\='right."
+  (let* ((origin (mu4e-walk--email-address-at-point))
+         (origin-start (when origin (plist-get origin :start)))
+         (origin-end (when origin (- (plist-get origin :end) 1)))
+         (point (point))
+         (target-start nil)
+         (target-end nil)
+         (search-function nil))
+    (save-excursion
+      (when (eq direction 'up)
+        (forward-line -1)
+        (end-of-line)
+        (setq point (point))
+        (setq direction 'left))
+      (when (eq direction 'down)
+        (forward-line 1)
+        (beginning-of-line)
+        (setq point (point))
+        (setq direction 'right))
+      (when (eq direction 'left)
+        (setq search-function 're-search-backward))
+      (when (eq direction 'right)
+        (setq search-function 're-search-forward))
+      (when search-function
+        (cl-loop
+         while (funcall search-function "@" nil t)
+         do (let ((target (mu4e-walk--email-address-at-point)))
+              (when (and target
+                         (not (eq origin-start
+                                  (plist-get target :start))))
+                (setq target-start (plist-get target :start))
+                (setq target-end (plist-get target :end))
+                (cl-return t))))))
+    (cond ((and origin (region-active-p)
+                (eq direction 'right))
+           (cond ((< point origin-end)
+                  (goto-char origin-end))
+                 ((and (= point origin-end) target-start)
+                  (goto-char target-start))))
+          ((and origin (region-active-p)
+                (eq direction 'left))
+           (cond ((> point origin-start)
+                  (goto-char origin-start))
+                 ((and (= point origin-start) target-end)
+                  (goto-char target-end))))
+          ((and origin
+                (> point origin-start)
+                (< point origin-end))
+           (progn
+             (when (eq direction 'left) (goto-char origin-start))
+             (when (eq direction 'right) (goto-char origin-end))))
+          ((and origin
+                (eq direction 'right)
+                (= point origin-start))
+           (goto-char origin-end))
+          ((and origin
+                (eq direction 'left)
+                (= point origin-end))
+           (goto-char origin-start))
+          (target-start
+           (progn
+             (if (eq direction 'right) (goto-char (- target-end 1)) (goto-char target-start))
+             (when mu4e-walk-use-overlays
+               (let ((overlay (make-overlay target-start target-end)))
+                 (overlay-put overlay 'face '(:box (:line-width 1)))
+                 (run-at-time "0.5 sec" nil
+                              (lambda () (delete-overlay overlay))))))))))
+
+;;;###autoload
+(defun mu4e-walk-switch-up ()
+  "Switch to next email address in the line above."
+  (interactive)
+  (mu4e-walk--switch 'up))
+
+;;;###autoload
+(defun mu4e-walk-switch-down ()
+  "Switch to next email address in the line below."
+  (interactive)
+  (mu4e-walk--switch 'down))
+
+;;;###autoload
+(defun mu4e-walk-switch-left ()
+  "Switch to next email address on the left."
+  (interactive)
+  (mu4e-walk--switch 'left))
+
+;;;###autoload
+(defun mu4e-walk-switch-right ()
+  "Switch to next email address on th right."
+  (interactive)
+  (mu4e-walk--switch 'right))
+
+
+;;====================
+;;
+;; Deleting
+;;
+;;--------------------
+
+(defun mu4e-walk-delete-email-address-at-point ()
+  "Delete email address at point."
+  (interactive)
+  (let* ((email (mu4e-walk--email-address-at-point))
+         (email-start (plist-get email :start))
+         (email-end (plist-get email :end))
+         (line-number (line-number-at-pos)))
+    (when email
+      (when (region-active-p) (setq deactivate-mark nil))
+      (delete-region email-start email-end)
+      (mu4e-walk--clean-address-field-at-point)
+      (goto-char email-start)
+      (when (< line-number (line-number-at-pos))
+        (forward-line -1)
+        (end-of-line)
+        (when (mu4e-walk--email-address-at-point) (backward-char))))))
+
+
 ;;====================
 ;;
 ;; Keybindings
@@ -243,7 +383,7 @@ DIRECTION can be \\='up, \\='down, \\='left, \\='right."
     (keymap-local-set key
                       `(lambda ()
                          (interactive)
-                         (if (message-point-in-header-p)
+                         (if (mu4e-walk--point-in-address-field-p) ; Exclude subject line
                              (call-interactively (quote ,walk-fun))
                            (call-interactively (quote ,oldfun)))))))
 
@@ -253,7 +393,12 @@ DIRECTION can be \\='up, \\='down, \\='left, \\='right."
   (mu4e-walk-extend-key "M-<up>" 'mu4e-walk-up)
   (mu4e-walk-extend-key "M-<down>" 'mu4e-walk-down)
   (mu4e-walk-extend-key "M-<left>" 'mu4e-walk-left)
-  (mu4e-walk-extend-key "M-<right>" 'mu4e-walk-right))
+  (mu4e-walk-extend-key "M-<right>" 'mu4e-walk-right)
+  (mu4e-walk-extend-key "C-<up>" 'mu4e-walk-switch-up)
+  (mu4e-walk-extend-key "C-<down>" 'mu4e-walk-switch-down)
+  (mu4e-walk-extend-key "C-<left>" 'mu4e-walk-switch-left)
+  (mu4e-walk-extend-key "C-<right>" 'mu4e-walk-switch-right)
+  (mu4e-walk-extend-key "M-<delete> M-<delete>" 'mu4e-walk-delete-email-address-at-point))
 
 ;; (add-hook 'mu4e-compose-mode-hook 'mu4e-walk-add-keybindings-compose-mode)
 
